@@ -346,6 +346,7 @@ static const iBlock *replaceVariables_(const iBlock *block) {
             if (equal_Rangecc(name, "APP_VERSION")) {
                 repl = range_CStr(LAGRANGE_APP_VERSION);
             }
+#if 0
             else if (startsWith_Rangecc(name, "BT:")) { /* block text */
                 repl = range_String(collect_String(renderBlockChars_Text(
                     &fontFiraSansRegular_Embedded,
@@ -356,12 +357,13 @@ static const iBlock *replaceVariables_(const iBlock *block) {
             }
             else if (startsWith_Rangecc(name, "ST:")) { /* shaded text */
                 repl = range_String(collect_String(renderBlockChars_Text(
-                    &fontSymbola_Embedded,
+                    &fontSmolEmojiRegular_Embedded,
                     20,
                     shading_TextBlockMode,
                     &(iString){ iBlockLiteral(
                         name.start + 3, size_Range(&name) - 3, size_Range(&name) - 3) })));
             }
+#endif
             else if (equal_Rangecc(name, "ALT")) {
 #if defined (iPlatformApple)
                 repl = range_CStr("\u2325");
