@@ -188,6 +188,7 @@ static void clear_Keys_(iKeys *d) {
 enum iBindFlag {
     argRepeat_BindFlag  = iBit(1),
     argRelease_BindFlag = iBit(2),
+    noDirectTrigger_BindFlag = iBit(3), /* can only be triggered via LabelWidget */
 };
 
 /* TODO: This indirection could be used for localization, although all UI strings
@@ -227,7 +228,16 @@ static const struct { int id; iMenuItem bind; int flags; } defaultBindings_[] =
     { 81, { "${keys.tab.next}",             nextTab_KeyShortcut,            "tabs.next"                         }, 0 },
     { 90, { "${keys.split.menu}",           SDLK_j, KMOD_PRIMARY,           "splitmenu.open"                    }, 0 },
     { 91, { "${keys.split.next}",           SDLK_TAB, KMOD_CTRL,            "keyroot.next",                     }, 0 },
+    { 92, { "${keys.split.item} ${menu.split.merge}",           '1', 0,     "ui.split arg:0",                   }, noDirectTrigger_BindFlag },
+    { 93, { "${keys.split.item} ${menu.split.swap}",            SDLK_x, 0,  "ui.split swap:1",                  }, noDirectTrigger_BindFlag },
+    { 94, { "${keys.split.item} ${menu.split.horizontal}",      '3', 0,     "ui.split arg:3 axis:0",            }, noDirectTrigger_BindFlag },
+    { 95, { "${keys.split.item} ${menu.split.horizontal} 1:2",  SDLK_d, 0,  "ui.split arg:1 axis:0",            }, noDirectTrigger_BindFlag },
+    { 96, { "${keys.split.item} ${menu.split.horizontal} 2:1",  SDLK_e, 0,  "ui.split arg:2 axis:0",            }, noDirectTrigger_BindFlag },
+    { 97, { "${keys.split.item} ${menu.split.vertical}",        '2', 0,     "ui.split arg:3 axis:1",            }, noDirectTrigger_BindFlag },
+    { 98, { "${keys.split.item} ${menu.split.vertical} 1:2",    SDLK_f, 0,  "ui.split arg:1 axis:1",            }, noDirectTrigger_BindFlag },
+    { 99, { "${keys.split.item} ${menu.split.vertical} 2:1",    SDLK_r, 0,  "ui.split arg:2 axis:1",            }, noDirectTrigger_BindFlag },
     { 100,{ "${keys.hoverurl}",             '/', KMOD_PRIMARY,              "prefs.hoverlink.toggle"            }, 0 },
+    { 110,{ "${menu.save.downloads}",       SDLK_s, KMOD_PRIMARY,           "document.save"                     }, 0 },
     /* The following cannot currently be changed (built-in duplicates). */
 #if defined (iPlatformApple)
     { 1002, { NULL, SDLK_LEFTBRACKET, KMOD_PRIMARY,     "navigate.back"                 }, 0 },
@@ -301,7 +311,10 @@ static iBinding *findCommand_Keys_(iKeys *d, const char *command) {
 static void updateLookup_Keys_(iKeys *d) {
     clear_PtrSet(&d->lookup);
     iConstForEach(Array, i, &d->bindings) {
-        insert_PtrSet(&d->lookup, i.value);
+        const iBinding *bind = i.value;
+        if (~bind->flags & noDirectTrigger_BindFlag) {
+            insert_PtrSet(&d->lookup, i.value);
+        }
     }
 }
 
