@@ -460,6 +460,7 @@ iBool handleCommand_Translation(iTranslation *d, const char *cmd) {
     if (equalWidget_Command(cmd, w, "translation.finished")) {
         if (!isFinished_Translation(d)) {
             if (processResult_Translation_(d)) {
+                setupSheetTransition_Mobile(d->dlg, iFalse);
                 destroy_Widget(d->dlg);
                 d->dlg = NULL;
             }
@@ -472,10 +473,11 @@ iBool handleCommand_Translation(iTranslation *d, const char *cmd) {
             updateTextCStr_LabelWidget(
                 findMenuItem_Widget(findChild_Widget(d->dlg, "dialogbuttons"),
                                     "translation.cancel"),
-                "${dismiss}");
+                "${close}");
             cancel_TlsRequest(d->request);
         }
         else {
+            setupSheetTransition_Mobile(d->dlg, iFalse);
             destroy_Widget(d->dlg);
             d->dlg = NULL;
         }
